#!/usr/bin/python
#
#  BIND configuration module  
# 
#  DNS Security Utilities
#
# Jason Vas Dias, Red Hat Inc., January 2005

import sys
import os
import fcntl
import termios
import time
import signal
import commands
import datetime
import base64
import string
import copy

import gtk
import gtk.glade
import gnome.ui
import gnome
import gobject
import pango
if (__name__ == "__main__") or (__name__=='DNSsec'):
    import __builtin__
    import gettext
    os.environ['SCB']='/usr/share/system-config-bind'
    os.putenv('SCB','/usr/share/system-config-bind')
    SCB='/usr/share/system-config-bind'
    gettext.bindtextdomain('bindconf', '/usr/share/locale')
    gettext.textdomain('bindconf')
    __builtin__.__dict__['_'] = gettext.gettext
    import Debug
    dbg = os.getenv('SCB_DEBUG')
    if dbg != None:
        Debug.DEBUG_CWD = SCB
        Debug.DEBUG = 1
        Debug.parseEnv(dbg,Debug)
import DNS
import RR
import Zone

def keytag(secret, flags, protocol, algorithm ) :
    key=base64.decodestring(secret)
    ac=0
    i=0
    key += chr((flags >> 8) & 0xff) + chr(flags & 0xff) + chr(protocol & 0xff) + chr(algorithm & 0xff)  
    for c in key:
        ac += (ord(c) << (((i&1)^1)*8)) & 0xffff
        i  += 1
    return ((ac + (ac >> 16)) & 0xffff)

def KeyTag( key ):
    if key.__class__ not in [ RR.KEY, RR.DNSKEY ]:
        return None
    return keytag( str(key.KEY), key.FLAGS.flags, key.PROTOCOL.protocol, key.ALGORITHM.algorithm )

def KeyID(key):
    return(   'K' + key.zone.zone_origin + '+'
            + ('%0.3u' % key.ALGORITHM.algorithm) + '+'
            + ('%0.5u' % KeyTag(key))              
          )

def KeyFile( key ):
    if not key.__class__ in [ RR.KEY, RR.DNSKEY ]:
        return None
    return(   'K' + key.zone.zone_origin + '+'
            + ('%0.3u' % key.ALGORITHM.algorithm) + '+'
            + ('%0.5u' % KeyTag(key))
            + '.key'
          )

def KeyPrivateFile( key ):
    if not key.__class__ in [ RR.KEY, RR.DNSKEY ]:
        return None
    return(   'K' + key.zone.zone_origin + '+'
            + ('%0.3u' % key.ALGORITHM.algorithm) + '+'
            + ('%0.5u' % KeyTag(key))
            + '.private'
          )

def SaveKey( bind, key ):
    keyFN = bind.nmdc.directory() + '/' + KeyFile ( key )
    try:
        keyFile = file( keyFN, "w" )
        keyFile.write( key.out() )
        keyFile.flush()
        keyFile.close()
        key.keyFile = keyFN
        commands.getstatus( '/bin/chmod 600 '+keyFN+'; /usr/bin/chcon dnssec_t '+keyFN )
        debug(  'SAVED KEY', keyFN )
    except IOError:
        pass

def ReadKey( keyFile, zone ):
    key = None
    debug(  'ReadKey:', keyFile, zone.zone_origin )
    try:
        f = file(keyFile,"r")
        key_line = f.readline()
        key_line.rstrip("\n")
        keyTokens = key_line.split()
        try:
            rrTypeIndex = keyTokens.index('DNSKEY')
        except ValueError:
            try:
                rrTypeIndex = keyTokens.index('KEY')
            except ValueError:
                rrTypeIndex = -1
        if rrTypeIndex == -1:
            return None
        if keyTokens[ rrTypeIndex ] == 'DNSKEY':
            key = RR.DNSKEY( zone, DNS.Name(zone.zone_origin,zone), 'IN', None, keyTokens, [], rrTypeIndex + 1 )
        else:
            key = RR.KEY( zone, DNS.Name(zone.zone_origin, zone), 'IN', None, keyTokens, [], rrTypeIndex + 1 )
        key.keyFile = keyFile
        key.keyHasPrivateFile = ( os.access( keyFile[:-4]+'.private', os.F_OK) == 1 )
        f.close()
    except IOError:
        pass
    debug(  'READ KEY ', keyFile, 'OK:', (key != None) )
    return key
    
def FindZoneKeys( bind, zone, otherDir = None ):
    if(otherDir == None):
        dir = bind.nmdc.directory()
    else:
        dir = otherDir
    cmd = 'find '+ dir +" -name 'K"+ zone.zone_origin + '+*.key'+"'"
    debug(  'FindZoneKeys:',cmd )
    s,out = commands.getstatusoutput(cmd)
    keys=[]
    if s != 0:
        return []
    else:
        for fn in out.split('\n'):
            debug(  '  FOUND KEY:', fn )
            k = ReadKey(fn,zone)
            if( (k != None) and not
                ( ((k.FLAGS.flags & DNS.KeyFlags.Mnemonics['SEP']) ==DNS.KeyFlags.Mnemonics['SEP']) and
                  not hasattr(k,'keyPrivateFile')
                )
              ):
                keys.append(k)
    return keys                            

def GenerateKSKCommand(bind, zone, size=1024):
    return ( 'cd '+ bind.nmdc.directory()
            +'; /usr/sbin/dnssec-keygen -r/dev/random -fKSK -aRSASHA1 -b'
            + str(size) + ' -nZONE ' + zone.zone_origin
           )

def GenerateKSK(bind, zone, size = 1024, errors=None ):
    r,out  = commands.getstatusoutput(GenerateKSKCommand(bind,zone, size))
    if r != 0:
        if type(errors) == list :
            errors.append( out.split() )
        return None
    fn = out.rstrip("\n")+'.key'
    return ReadKey( fn, zone )

def GenerateZSKCommand(bind, zone, size=1024 ):
    return ( 'cd '+ bind.nmdc.directory()
            +'; /usr/sbin/dnssec-keygen -r/dev/random -aRSASHA1 -b'
            + str(size) + ' -nZONE ' + zone.zone_origin
           )

def GenerateZSK(bind, zone, size = 1024 ):
    r,out  = commands.getstatusoutput(GenerateZSKCommand(bind,zone, size))
    if r != 0:
        if type(errors) == list :
            errors.append( out.split() )        
        return None
    fn = out.rstrip("\n")+'.key'
    return ReadKey( fn, zone )

def KeyGenerationDialogPulse( info ):
    info.progressBar.pulse()
    w,h = info.doneDialog.dialog.get_size()
    x,y = info.doneDialog.dialog.get_position()
    if info.keyName == 'ksk':
       myw,myh=info.dialog.get_size()
       info.dialog.move ( x-myw, h+y )
    else:
       info.dialog.move ( x+w, h+y)
    gobject.source_remove(info.sid)
    pid,status=(None,None)
    try:
        pid, status = os.waitpid( info.pid, os.WNOHANG )
    except OSError:
        info.doneDialog.done(info.doneDialog, info.keyName, -1)        
    if ( pid == info.pid ):
        errors=-1
        if status == 0:
            keyID=None
            try:
                f=file('/tmp/'+info.keyName,'r')
                keyID=f.readline()
                keyID = keyID.rstrip('\n')
                f.close()
                os.unlink('/tmp/'+info.keyName)
            except OSError:
                pass
            except IOError:
                pass
            if( keyID != None ):
                key = ReadKey( info.bind.nmdc.directory() + '/' +  keyID +'.key', info.zone )
                if( key != None ):
                    setattr( info.doneDialog, info.keyName, key )
                    key.update()
                    info.dialog.hide()
                    info.dialog.destroy()
                    setattr(info.doneDialog, info.keyName +'Dialog', None)
                    try:
                        os.unlink('/tmp/'+info.keyName+'ERR')
                    except:
                        pass
                    info.doneDialog.done( info.doneDialog, info.keyName, None )
                    return
                else:
                    debug(  'ReadKey Failed!' )
        else:
            errors=status
            debug(  'keygen command failed:',status )
        setattr( info.doneDialog, info.keyName+'PID', pid) 
        try:
            f=file('/tmp/'+info.keyName+'ERR','r')
            errors=f.read()
            f.close()
        except OSError:
            pass
        except IOError:
            pass
        info.doneDialog.done( info.doneDialog, info.keyName,errors )
        try:
            os.unlink('/tmp/'+info.keyName+'ERR')
        except OSError:
            pass
        info.dialog.hide()
        info.dialog.destroy()
        setattr(info.doneDialog, info.keyName +'Dialog', None)                    
    else:
        info.sid=gobject.timeout_add(1000, KeyGenerationDialogPulse, info )
                                  
def KeyGenerationDialog( bind, zone, genCmdFunc, title, doneDialog, keyName ):
    info  = bind.gui.info(bind.gui.gui,_('Generating DNSSEC Key'),
                         _('Generating %s\nfor zone: %s') % ( title, zone.zone_origin )
                         )
    info.progressBar = gtk.ProgressBar()
    info.progressBar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
    info.progressBar.set_fraction(0.0)
    info.progressBar.set_pulse_step(0.1)
    info.dialog.vbox.pack_start( info.progressBar )
    info.dialog.set_response_sensitive(gtk.RESPONSE_OK, False)
    info.dialog.resize_children()
    info.dialog.show_all()
    info.pid = os.fork() 
    if( info.pid == 0 ):
        cmd =  genCmdFunc(bind, zone)+' >/tmp/'+keyName +' 2>/tmp/'+keyName+'ERR'
        debug(  keyName, 'Generate Cmd:', cmd )
        os.execl( '/bin/bash', '', '-c', cmd)
    info.bind = bind
    info.zone = zone
    info.doneDialog = doneDialog
    debug(  'DONE DIALOG:', doneDialog )
    setattr(info.doneDialog, keyName +'Dialog', info)
    info.keyName = keyName    
    info.sid=gobject.timeout_add(1000, KeyGenerationDialogPulse, info )
    return info

def EnableDNSSEC(bind):
    if(  (bind.version >= '9.3.0') and
      not( (not((bind.nmdc.__contains__(['options','dnssec-enable']))))
         or(bind.nmdc['options','dnssec-enable'].lower() != 'yes')
         )
      ):
        if not (bind.nmdc.__contains__(['options','dnssec-enable'])):            
            bind.nmdc['options','dnssec-enable'] = 'yes;\n'
        else:
            bind.nmdc['options','dnssec-enable'] = 'yes'
        bind.nmdc.update()
        bind.nmdc.save()
        return (len(bind.nmdc.errors)>0,bind.nmdc.errors)
    return (0,'')
    
def SignZoneWithKeys( bind, zone, ksk, zsk ):
    e,s = commands.getstatusoutput(  'cd ' + bind.nmdc.directory() +'; ' 
                                    + '/usr/sbin/dnssec-signzone -r/dev/random -o'
                                    + zone.zone_origin  + ' '
                                    + '-k' + ksk.keyFile[:-4] + ' '
                                    + zone.filename + ' '
                                    + zsk.keyFile
                                   )
    if( e == 0 ):
        f = zone.filename + '.signed'
        if os.access(f, os.F_OK ) == 1:
            debug(  'SIGNED:', f, 'UNSIGNED:', zone.filename )
            if (len(zone.filename)>7):
                debug(  zone.filename[-7:], (zone.filename[-7:] == '.signed') )
            if (len(zone.filename)>7) and  (zone.filename[-7:] == '.signed'):
                debug(  'RE-SIGNED FILE' )
                d=datetime.datetime.now()
                ts=str(d.year)+'-'+str(d.month)+'-'+str(d.day)+'_'+str(d.hour)+'.'+str(d.minute)+'.'+str(d.second)
                try:
                    os.rename( zone.filename, zone.filename + '.' + ts)
                    os.rename( f, zone.filename )
                    f = f[:-7]
                except :
                    pass
            else:                
                dir = bind.nmdc.directory() + '/'
                if (len(f)>len(dir)) and (f[:len(dir)] == dir):
                    f = f[len(dir):]            
                bind.nmdc[ zone.nmdcKey + [ 'file' ] ] = '"' + f + '"'
                bind.nmdc.errors=[]            
                bind.nmdc.update()
            EnableDNSSEC(bind)
            if bind.nmdc.dirty:
                bind.nmdc.save()
            if( len(bind.nmdc.errors) == 0 ):
                bind.load()
                return (0,'')
            else:
                return(1, bind.nmdc.errors)
        else:
            return (1,_('Signed zone file')+'\n'+f+'\n'+_('was not found after dnssec-keygen completed.'))
    else:
        return (e,s)

def SignZoneDialogDone( info, keyName, error ):
    info.progressBar.pulse()
    gobject.source_remove(info.sid)    
    if ( error != None ):
        info.errorText.set_text('Zone Signing Failed:\n' + keyName + 'Key Generation Failed:\n' + str(error))
        info.dialog.resize_children()
        info.dialog.show_all()
        if  hasattr( info, 'kskPID'):
            try:
                os.kill(info.kskPID,signal.SIGTERM)
            except OSError:
                pass
        if  hasattr( info, 'zskPID'):
            try:
                os.kill(info.zskPID,signal.SIGTERM)
            except OSError:
                pass
        info.dialog.set_response_sensitive(gtk.RESPONSE_OK, True)
        kd = None
        try:
            kd = getattr(info, keyName +'Dialog')
        except:
            pass
        if (kd != None):
            try:
                gobject.source_remove(kd.sid)
                kd.dialog.hide()
                kd.dialog.destroy()
            except:
                pass
        return
    if ( info.ksk != None ) and (info.zsk != None):
        txt = info.errorText.get_text()
        if ( keyName != '' ):
            key = getattr(info, keyName)
            if key != None:
                dir = info.bind.nmdc.directory() + '/'
                id = key.keyFile[len(dir):]
                id = id[:-4]
                txt += '\n' +  _('Generated %s %s key OK.') % (keyName, id)
                info.errorText.set_text(txt)
                info.dialog.resize_children()
                info.dialog.show_all()
        txt += '\n' + _('Signing Zone...')
        info.errorText.set_text(txt)
        info.dialog.resize_children()
        info.dialog.show_all()        
        if info.zone.dirty:
            info.zone.save()
            if len(info.zone.errors):
                info.errorText.set_text(txt + '\n' +_('Save of Zone with new keys failed:\n%s') % string.join(zone.errors,"\n"))
                info.dialog.set_response_sensitive(gtk.RESPONSE_OK, True)                            
                info.dialog.resize_children()
                info.dialog.show_all()
                return
        status, out = SignZoneWithKeys(info.bind, info.zone, info.ksk, info.zsk)
        if( status == 0 ):
            info.errorText.set_text(txt + '\n' + _('Zone Signing Complete.'))
        else:
            info.errorText.set_text(txt + '\n' + (_('Zone Signing Failed:\n%s:\n') % str(status)) + out)
        if (not hasattr(bind,'onExit')) or not bind.onExit:
            info.dialog.set_response_sensitive(gtk.RESPONSE_OK, True)                            
            info.dialog.resize_children()        
            info.dialog.show_all()
        else:
            time.sleep(2)
            info.dialog.hide()
            info.dialog.destroy()
    else:
        if hasattr(info, keyName ):
            key = getattr(info, keyName)
            if key != None:
                txt = info.errorText.get_text()
                dir = info.bind.nmdc.directory() + '/'
                id = key.keyFile[len(dir):]
                id = id[:-4]
                txt += '\n'+  _('Generated %s %s key OK.\n') % (keyName, id)
                info.errorText.set_text(txt)
                info.dialog.resize_children()
                info.dialog.show_all()
        info.sid = gobject.timeout_add(1000, SignZoneDialogDone, info,  '', None)
    
def SignZoneDialog( bind, zone ):
    info  = bind.gui.info(bind.gui.gui,_('DNSSEC Zone Security'),
                         _('Signing zone') + ' ' + zone.zone_origin
                         )
    info.progressBar = gtk.ProgressBar()
    info.progressBar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
    info.progressBar.set_fraction(0.0)
    info.progressBar.set_pulse_step(0.1)
    info.dialog.vbox.pack_start( info.progressBar )
    info.dialog.set_response_sensitive(gtk.RESPONSE_OK, False)
    info.dialog.resize_children()
    info.ksk = None
    info.zsk = None
    info.zone = zone
    info.bind = bind
    info.done = SignZoneDialogDone    
    info.dialog.show_all()
    info.sid=gobject.timeout_add(1000, SignZoneDialogDone, info,  '', None)
    return info

def SignZone( bind, zone, regenerate=False, size=1024, start=None, end=None  ):
    zoneKeys=[]
    zoneKeyIDs={}
    debug(  'SignZone:', zone.zone_origin )
    ksk = None
    zsk = None                
    if not regenerate:
        # find existing zone keys:
        for r in zone.records:
            if r.Type() in [ 'KEY', 'DNSKEY']:
                f = KeyFile(r)
                if os.access( bind.nmdc.directory() + '/' + f, os.F_OK ) == 1:
                    r.keyFile = f
                    f = f[:-4]+'.private'
                    if os.access( bind.nmdc.directory() + '/' + f, os.F_OK ) == 1:
                        r.keyPrivateFile = f
                        zoneKeys.append(r)
                        tag = KeyTag(r)
                        debug(  'FOUND ZONE KEY: ', tag, ((r.FLAGS.flags & DNS.KeyFlags.Mnemonics[ 'SEP' ])>0) )
                        zoneKeyIDs[ tag ] = r
        if len(zoneKeys) == 0:
            # No keys in zone. Key files ?
            debug(  'NO KEYS IN ZONE - finding KEY files' )
            zoneKeys=FindZoneKeys( bind, zone )
            for key in zoneKeys:
                if ((key.FLAGS.flags & DNS.KeyFlags.Mnemonics[ 'SEP' ]) and
                    hasattr(key,'keyFile') and hasattr(key,'keyPrivateFile') and
                    key.keyPrivateFile) :
                    if ( ksk == None ):
                        ksk = key
                    else:
                        st_n=stat(key.keyFile)
                        st_c=stat(ksk.keyFile)
                        if st_c.st_mtime <  st_n.st_mtime:
                            ksk=key
                if (key.FLAGS.flags & DNS.KeyFlags.Mnemonics[ 'ZONE' ] and
                    hasattr(key,'keyFile')
                   ):
                    if( zsk == None ):
                        zsk = key
                    else:
                        st_n=stat(key.keyFile)
                        st_c=stat(zsk.keyFile)
                        if st_c.st_mtime <  st_n.st_mtime:
                            zsk=key
        else:
            # Keys in zone - find  ksk/zsk to use for signing
            unknownKeyIDs={}
            for r in zone.records:
                t = r.Type()
                if t in [ 'RRSIG', 'SIG' ]:
                    tag = r.KEYTAG.tag                        
                    if zoneKeyIDs.has_key( tag ):
                        debug(  r.TYPE, ( str(r.TYPE) in [ 'DNSKEY' , 'KEY' ] ),  (zoneKeyIDs[ tag ].FLAGS.flags & DNS.KeyFlags.Mnemonics['SEP']) == DNS.KeyFlags.Mnemonics['SEP'] )
                        if(   ( str(r.TYPE) in [ 'DNSKEY' , 'KEY' ] )
                          and ((zoneKeyIDs[ tag ].FLAGS.flags & DNS.KeyFlags.Mnemonics['SEP'])
                               == DNS.KeyFlags.Mnemonics['SEP']
                              )
                          ):
                            debug(  '  GOT ZONE KSK:', tag )
                            if ( ksk == None ):                            
                                ksk = zoneKeyIDs[ tag ]
                                ksk.incept = r.INCEPT.data
                            elif r.INCEPT.data > ksk.incept:
                                ksk = zoneKeyIDs[ tag ]
                        else:                            
                            debug(  '  GOT ZONE ZSK:', tag )
                            if ( zsk == None ):                            
                                zsk = zoneKeyIDs[ tag ]
                                zsk.incept = r.INCEPT.data
                            elif r.INCEPT.data > zsk.incept:
                                zsk = zoneKeyIDs[ tag ]
                    else:
                        unknownKeyIDs[ tag ] = r
                        debug(  'UNKNOWN KEY: ', tag )
    if hasattr( bind, 'gui' ) and bind.gui.running:
        signZoneDialog = SignZoneDialog(bind, zone)
        if ksk == None:
            KeyGenerationDialog( bind, zone, GenerateKSKCommand, _('Key Signing Key'), signZoneDialog, 'ksk' )
        else:
            signZoneDialog.ksk = ksk
        if zsk == None:
            KeyGenerationDialog( bind, zone, GenerateZSKCommand, _('Zone Signing Key'), signZoneDialog, 'zsk')
        else:
            signZoneDialog.zsk = zsk
        return
    if ksk == None:
        ksk = GenerateKSK( bind, zone )
        if ( ksk == None ):
            return [-1,_('KSK Generation Failed')]
        ksk.update()
    if zsk == None:
        zsk = GenerateZSK( bind, zone )
        if ( zsk == None ):
            return [-1,_('ZSK Generation Failed')]
        zsk.update()
    if zone.dirty:
        zone.save()
        if( len(zone.errors) ):
            return [-1, zone.errors]
    return SignZoneWithKeys( bind, zone, ksk, zsk )
    
def QueryZoneKeys(origin, server, error=None):
    search=commands.getstatusoutput('/bin/grep search /etc/resolv.conf | /bin/sed "s/search //"')[1]    
    digOpts='+noall +nocmd +nocomments +answer +tries=2 +time=1 +search='+"'"+search+"'"
    digCmd='/usr/bin/dig @'+server+' '+origin+' DNSKEY '+digOpts+' @'+server+' '+origin+' KEY '+digOpts
    s,ans=commands.getstatusoutput(digCmd)
    if ( s != 0 ):
        debug('dig failed:',ans)
        if( type(error) == list):
            error.append(ans)
        return []
    s=ans.rstrip('\n')
    s=s.strip()
    if (s == ''):
        debug('NO ANSWER')
        if( type(error) == list):
            error.append(_('Empty answer to query.'))
        return []
    debug('QueryZoneKeys:', ans)
    zone=Zone.Zone(None, origin, Zone.NullNmdc(), ['zone',origin])
    zone.contents=ans
    zone.parse()
    return zone.records

def quietSSH( server, sshCmd, status=None):
    r=file('/dev/random','r')
    outFile = '/tmp/' + base64.encodestring( r.read(6) ).rstrip('\n')
    r.close()
    pid=os.fork()
    if(pid==0):
        try:
            if (os.isatty(0)):
                try:
                    fcntl.ioctl(0, termios.TIOCNOTTY)
                except:
                    pass
            if (os.isatty(1)):
                try:
                    fcntl.ioctl(1, termios.TIOCNOTTY)
                except:
                    pass
            if (os.isatty(2)):
                try:
                    fcntl.ioctl(2, termios.TIOCNOTTY)
                except:
                    pass
            cmd = '/usr/bin/ssh -e none -f ' + server +" '" + sshCmd + "'"+ ' </dev/null 2>/dev/null >'+outFile
            os.execl('/bin/bash','','-c',cmd)
        except:
            sys.exit(1)
    wp=0
    s=0
    while(wp==0):
        wp,s=os.waitpid(pid,os.WNOHANG);
        time.sleep(1)
    out=''
    s>>=8
    if s==0:
        try:
            f=file(outFile,'r')
            out=f.read()
            f.close()
        except OSError, e:
            out=''
    try:
        os.unlink(outFile)
    except:
        pass
    if type(status)== list:
        status.append(s)
    return out

def sshGetKeys(server, origin):
    sshCmd='(set -e; ROOTDIR=""; if [ -f /etc/sysconfig/named ]; then . /etc/sysconfig/named; fi; if ! [ -d $ROOTDIR/var/named ] ; then exit 1; fi; cd $ROOTDIR/var/named; /bin/cat K'+origin+'+*.key;) 2>/dev/null'
    ans=quietSSH(server,sshCmd)
    zone=Zone.Zone(None, origin, Zone.NullNmdc(), ['zone',origin])
    zone.contents=ans
    zone.parse()
    for k in zone.records:
        k.name=DNS.Name(zone.zone_origin,zone)
        k.name.zone_origin=''
        k.clas= zone.clas
        k.ttl = None
        k.offsets=[]
    return zone.records

def sshPutKeys(server, zone, keys):
    sshCmd='(set -e; ROOTDIR=""; if [ -f /etc/sysconfig/named ]; then . /etc/sysconfig/named; fi; if ! [ -d $ROOTDIR/var/named ] ; then exit 1; fi; cd $ROOTDIR/var/named;'
    for k in keys:
        key=copy.deepcopy(k)
        key.name=DNS.Name(zone.zone_origin,zone)
        key.name.zone_origin=''
        key.offsets=[]
        key.clas =zone.clas
        key.ttl = None
        sshCmd += 'echo '+ "'" + key.out() + "'" + '>' + KeyFile(k) + ';'
    sshCmd+=')2>/dev/null'
    quietSSH(server,sshCmd)

class TrustedKeys:
    def __init__(s, bind):
        s.bind = bind
        s.keys = []
        s.key = 'trusted-keys'
        if bind.nmdc.__contains__('trusted-keys'):
            tk = bind.nmdc['trusted-keys']

            # if we have only one key, add it to list
            if (type(tk) == dict):
                tk=[tk]    
            for lk in tk:
                for k in lk.keys():
                    origin = k.strip('"')
                    if(type(lk[k]) != list):
                        print _('Unrecognized trusted-keys entry in named.conf:'),lk[k]
                        continue
                    tks=[]
                    if( type(lk[k][0]) != list ):
                        tks=[ lk[k] ]
                    else:
                        tks=lk[k]
                    for toks in tks:
                        if( len(toks) != 4 ):
                             print _('Unrecognized trusted-keys entry in named.conf:'),toks
                             continue
                        zone=Zone.Zone(None, origin, Zone.NullNmdc(), ['zone',k])
                        zone.zone_origin=''
                        try:
                            s.keys.append(
                                RR.DNSKEY( zone, DNS.Name(origin,zone), 'IN', None,
                                           [ origin, 'IN', 'DNSKEY',
                                             int(toks[0]), int(toks[1]), int(toks[2]),toks[3].strip('"')
                                           ], None, 3
                                         )
                                         )
                            s.keys[-1].index = len(s.keys)-1
                        except ValueError:
                            print  _('Bad trusted-keys entry in named.conf:'), origin,toks
                            continue
    
    def out(self):
        s='trusted-keys {\n'
        for k in self.keys:
            s += '\t' + '"' + str(k.name) +'"' + '\t' + str(k.FLAGS.flags) + '\t' + str(k.PROTOCOL.protocol) + '\t' + str(k.ALGORITHM.algorithm) + '\t' + '"' + str(k.KEY) +'"'+';\n'
        s += '};\n'
        return s

    def response(s,d,r):
        if r==gtk.RESPONSE_OK:
            if s.bind.nmdc.__contains__('trusted-keys'):
                del s.bind.nmdc['trusted-keys']
            if len(s.keys)>0:
                s.bind.nmdc['trusted-keys']='\n'+s.out()
                s.bind.nmdc.update()
                s.bind.zoneTree.load()
        s.dialog.hide()
        s.dialog.destroy()
        
    def selected(s,tree):
        selection = tree.get_selection()
        if( selection != None ):
            model, selected = selection.get_selected()
            if (model != None) and  (selected != None):
                return (selected,) + model.get(selected, 3)
        return (None,None)

    def removeZone(s,b):
        (iter,key)=s.selected(s.zoneTree)
        if key != None:
            s.keys.pop(key.index)
            if key.index < len(s.keys):
                for k in s.keys[key.index:]:
                    k.index -= 1
            s.zoneList.remove(iter)
        else:
            s.bind.gui.modalError(s.dialog,_('Zone Remove Error'), _('You must select a zone key to remove.'))
        

    def importMethodChanged(s, mc):
        method = s.azMethodCombo.get_active()
        s.importEntryFrame.set_label_widget(s.importEntryLabels[method])
        s.azDialog.resize_children()
        s.azDialog.show_all()
        if (method == 2) and (len(s.originEntry.get_text())==0):
            s.originEntry.set_text(s.bind.nmdc.directory())
        
    def addZoneResponse(s,d,r):
        if( r == gtk.RESPONSE_OK ):
            method =  s.azMethodCombo.get_active()
            if (len(s.importEntry.get_text())==0) or (len(s.originEntry.get_text())==0):
                s.bind.gui.modalError(s.dialog,_('Zone Key Query Error'),
                                      _('You must enter non-empty %s and Zone Origin strings.') %
                                      s.importEntryItems[s.azMethodCombo.get_active()]
                                     )
                return False
            ks=[]
            if method == 0:
                error=[]
                ks = QueryZoneKeys(s.originEntry.get_text(),s.importEntry.get_text(),error)
                if ( (ks==None) or (len(ks)==0)):
                    s.bind.gui.modalError(s.dialog,_('Zone Key Query Error'),
                          _('This zone contained no keys.')+'\n'+string.join(error,'\n')+'\n'
                         +_('Did you enter the DNS server and origin correctly?')
                                         )
                    return False                
            elif method == 1:
                ks = sshGetKeys(s.importEntry.get_text(), s.originEntry.get_text())
                if ( (ks==None) or (len(ks)==0)):
                    s.bind.gui.modalError(s.dialog,_('Remote Key File SSH Transfer Failed'),
                                          _('Check that Remote Server %s has zone:\n%s\nkey files in its $ROOTDIR/var/named directory.') %
                                          ( s.importEntry.get_text(), s.originEntry.get_text() )
                                         )
                    return False                
            elif method == 2:
                ks = FindZoneKeys(s.bind,s.originEntry.get_text(),otherDir=s.importEntry.get_text())
                if ( (ks==None) or (len(ks)==0)):
                    s.bind.gui.modalError(s.dialog,_('Local Key File Import Failed'),
                                          _('Check that the directory %s contains zone:\n%s\nKey Files.') %
                                          ( s.importEntry.get_text() , s.originEntry.get_text() )
                                         )
                    return False                
            for k in ks:
                kt='ZSK'
                if k.FLAGS.flags & k.FLAGS.Mnemonics['SEP']:
                    kt='KSK'
                k.index = len(s.keys)                    
                s.keys.append(k)
                s.zoneList.append([str(k.name), kt, ('%0.5u' % KeyTag(k)), k])                
        s.azDialog.hide()
        s.azDialog.destroy()
        return False
        
    def addZone(s,b):
        s.azDialog = gtk.Dialog(parent=s.dialog,
                              title=_('Import Trusted Zone Key'),
                              buttons=
                              (gtk.STOCK_OK,     gtk.RESPONSE_OK,
                               gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL
                              )
                             )
        s.azFrame = gtk.Frame(_('Select Key Import Method'))        
        s.azMethodCombo = gtk.combo_box_entry_new_text()
        s.azMethodCombo.append_text(_('From DNS Server'))
        s.azMethodCombo.append_text(_('From Remote File (SSH)'))
        s.azMethodCombo.append_text(_('From Local File'))
        s.importEntryLabels = [ gtk.Label(_('Enter DNS Server Name')),
                                gtk.Label(_('Enter Remote Server Name')),
                                gtk.Label(_('Enter Local Directory'))
                              ]
        s.importEntryItems = [ _('DNS Server Name'),
                               _('Remote Server Name'),
                               _('Local Directory')
                             ]
        s.importEntryFrame = gtk.Frame()
        s.importEntryFrame.set_label_widget(s.importEntryLabels[0])
        s.importEntry = gtk.Entry()
        s.importEntryFrame.add(s.importEntry)
        s.azMethodCombo.set_active(0)
        s.azMethodCombo.connect('changed',s.importMethodChanged)
        s.azFrame.add(s.azMethodCombo)
        s.originFrame=gtk.Frame(_('Enter Origin of Zone Containing Keys:'))
        s.originEntry=gtk.Entry()
        s.originFrame.add(s.originEntry)
        s.azDialog.vbox.pack_start(s.azFrame)
        s.azDialog.vbox.pack_start(s.importEntryFrame)
        s.azDialog.vbox.pack_start(s.originFrame)
        s.azDialog.connect('response',s.addZoneResponse)
        s.azDialog.show_all()
        
    def EditDialog(s):
        s.dialog = gtk.Dialog(parent=s.bind.gui.gui,
                              title=_('DNS Trusted Keys'),
                              buttons=
                              (gtk.STOCK_OK,     gtk.RESPONSE_OK,
                               gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL
                              )
                             )
        s.dialog.connect('response',s.response)
        s.zoneFrame = gtk.Frame(_('Trusted Zones'))
        s.zoneHB = gtk.HBox()
        s.zoneList = gtk.ListStore( gobject.TYPE_STRING,  gobject.TYPE_STRING, gobject.TYPE_STRING,  gobject.TYPE_PYOBJECT )
        s.zoneTree = gtk.TreeView( model=s.zoneList )
        s.zoneTree.set_headers_visible(False)
        s.zoneCell = gtk.CellRendererText()
        s.zoneColm = gtk.TreeViewColumn( None, s.zoneCell, text=0 )
        s.kCell = gtk.CellRendererText()
        s.kColm = gtk.TreeViewColumn( None, s.kCell, text=1 )
        s.kidCell = gtk.CellRendererText()
        s.kidColm = gtk.TreeViewColumn( None, s.kCell, text=2 )        
        s.zoneTree.append_column(s.zoneColm)
        s.zoneTree.append_column(s.kColm)
        s.zoneTree.append_column(s.kidColm)        
        s.zoneSW = gtk.ScrolledWindow()
        s.zoneSW.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        s.zoneSW.add_with_viewport(s.zoneTree)
        s.zoneHB.pack_start(s.zoneSW)
        s.zoneVB=gtk.VBox()
        s.addButton = gtk.Button(stock=gtk.STOCK_ADD)
        s.addButton.connect('clicked',s.addZone)
        s.zoneVB.pack_start(s.addButton,False)
        s.removeButton=gtk.Button(stock=gtk.STOCK_REMOVE)
        s.removeButton.connect('clicked',s.removeZone)
        s.zoneVB.pack_start(s.removeButton,False)
        s.zoneHB.pack_start(s.zoneVB,False)
        s.zoneFrame.add(s.zoneHB)
        s.dialog.vbox.pack_start(s.zoneFrame)
        for k in s.keys:
            kt='ZSK'
            if k.FLAGS.flags & k.FLAGS.Mnemonics['SEP']:
                kt='KSK'
            s.zoneList.append([ str(k.Name()), kt, ('%0.5u' % KeyTag(k)), k ])
        s.dialog.resize_children()
        s.dialog.show_all()

                    

