#!/usr/bin/python
#
# BIND configuration GUI module
#
# Copyright (C) 2004  Red Hat, Inc. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Any Red Hat trademarks that are incorporated in the source code or
# documentation are not subject to the GNU General Public License
# and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Jason Vas Dias


import sys
import os
import __builtin__
import gettext

__builtin__.__dict__['SCB']='/usr/share/system-config-bind'
os.environ['SCB']='/usr/share/system-config-bind'
os.putenv('SCB','/usr/share/system-config-bind')
gettext.bindtextdomain('bindconf', '/usr/share/locale')
gettext.textdomain('bindconf')
__builtin__.__dict__['_'] = gettext.gettext

try:
    import gtk
    import gtk.glade
    import gnome.ui
    import gnome
except RuntimeError:
    sys.stderr.write(_("ERROR: Unable to initialize graphical environment.\n") + \
                     _("Most likely cause of failure is that the tool was not run using a graphical environment.\n") + \
                     _("Please either start your graphical user interface or set your DISPLAY variable.\n"))
    sys.exit(0)
import Debug
dbg = os.getenv('SCB_DEBUG')
if dbg != None:
    Debug.DEBUG_CWD = SCB
    Debug.DEBUG = 1
    Debug.parseEnv(dbg,Debug)
import DNS
import RRGUI
from BIND import *
from ErrorDialog import ErrorDialog

class GUI:
    fatal_error=False
    running=False
    
    def exit(s, answer=None, noparms=None):
        debug( 'GUI EXIT')
        if not(s.fatal_error) and s.bind.dirty() and (answer == None):
            ask = s.question(s.gui, _('DNS Configuration Modified'), _('The DNS Configuration has been modified.\n Do you really want to exit and discard your changes?'))
            ask.question(s.exit)
            return True
        else:
            if ( answer == None ) or (answer == gtk.RESPONSE_OK):
                debug( 'GUI EXITING')
                if s.running:
                    gtk.main_quit()
                    sys.exit()
            elif( answer == gtk.RESPONSE_CANCEL ):
                s.bind.zoneTree.SaveClicked(s.gui, onExit = True)
                return True
        return False
        
    def on_exit(s, arg):
        return s.exit()
    
    def delete_event(s, widget, event, data=None):
        return s.exit()
        
    def __init__(self,argv):
        gnome.program_init('bindconf', '1.0')
        gtk.glade.bindtextdomain('bindconf', "/usr/share/locale")
        # gtk2-2.6.0 now emits an annoying warning message:
        # "Mixing deprecated and non-deprecated GtkToolbar API is not allowed"
        # and there appears to be no way to prevent it (bug 144301).
        try:
                fd = open(SCB+'/'+'system-config-bind.version', 'r')
                __builtin__.__dict__['SCB_VER'] = (string.split(fd.readline()))[0]
                fd.close()
        except:
            __builtin__.__dict__['SCB_VER'] = '4.0.0'
        sys.stdout = file("/dev/null","w")
        sys.stderr = sys.stdout 
        self.xml = gtk.glade.XML ( SCB+'/'+'system-config-bind.glade', domain="bindconf")
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__
        self.gui = self.xml.get_widget('GUI')
        self.aboutDialog = self.xml.get_widget("AboutDialog")
        self.bind=BIND(self, argv)
        if self.fatal_error:
            return
        __builtin__.__dict__['bind'] = self.bind
        self.xml.signal_autoconnect({'on_exit': self.on_exit,
                                     'on_about': self.about_dialog,
                                     'on_manual': self.show_manual,
                                     'on_isc_arm': self.show_arm,
                                     'on_AboutDialog_close': self.about_dialog_close,
                                     'on_AboutDialog_response': self.about_dialog_response,
                                     'on_AboutDialog_delete_event': self.about_dialog_destroy,
                                     'on_AboutDialog_destroy_event': self.about_dialog_destroy
                                    }
                                   )
        self.gui.connect("delete_event", self.delete_event)
        self.gui.show_all()
        self.running = True
        gtk.main()
            
    def widget( s, obj ):
        return RRGUI.widget(RRGUI,obj)
    
    def error(self, title, text, fatal=False, warning=False):
        parent = self.gui
        if not self.running:
            parent = None
        return ErrorDialog(self,title,text,fatal,parent,warning)

    def modalError(self, parent, title, text, fatal=False, warning=False):
        return ErrorDialog(self,title,text,fatal,parent,warning)

    def warning(self, parent, title, text):
        return ErrorDialog(self,title,text,False,parent,warning=True)

    def info(self, parent, title, text):
        return ErrorDialog(self,title,text,False,parent,info=True)

    def question(self, parent, title, text, modal=gtk.DIALOG_MODAL):
        return ErrorDialog(self,title,text,False,parent,question=True,modal=modal) 

    def about_dialog(self, arg):
        self.aboutDialog.set_transient_for(self.gui)
        self.aboutDialog.show()

    def about_dialog_close(self, dialog, event=None):
        dialog.hide()
        debug( 'ABOUT CLOSE')
        return True

    def about_dialog_response(self, dialog, response):
        if response < 0:
            dialog.hide()
            dialog.emit_stop_by_name('response')
        debug( 'ABOUT CLOSE RESPONSE: ',response)
        return True
    
    def about_dialog_destroy(self, dialog, event=None):
        dialog.hide()
        debug( 'ABOUT DESTROY')
        return True

    def show_manual(self, dialog, event=None):
        url = "ghelp:system-config-bind#s1-system-config-bind"
        paths = ["/usr/bin/yelp", None]
        for path in paths:
            if path and os.access (path, os.X_OK):
                break
        if path == None:
            messageDialog.show_message_dialog (_("The help viewer could not be found. To be able to view help you need to install the 'yelp' package."))
            return
        pid = os.fork()
        if not pid:
            os.execv(path, [path, url])
        return

    def show_arm(self, dialog, event=None):
        try:
            pid = os.fork()
        except:
            return
        if (pid == 0):
            try:
                if os.access('/usr/bin/xdg-open', os.X_OK):
                    os.execl('/usr/bin/xdg-open','', '/usr/share/doc/bind-'+self.bind.version_main+'/arm/Bv9ARM.html')
            except:
                pass
            sys.exit(1)
        return

if __name__ == '__main__':
    GUI(sys.argv)
    
